#!/usr/bin/env node
var Project, commander, path, srv, _base;
srv = "server";
try {
  require.resolve("../server/fe");
} catch (e) {
  srv = "lib";
}
(_base = process.env).NODE_ENV || (_base.NODE_ENV = "development");
commander = require('commander');
Project = require("../" + srv + "/fe/models/project");
path = require('path');
commander.version('0.0.1').option('-l, --local', 'Restrict access to Stratus from localhost').option('-P, --pass <pass>', 'Authenticate with http basic auth stratus:<pass>');
commander.command('start [port]').description('  start the Stratus server').action(function(port) {
  var fe, options;
  options = {};
  if (commander.local) {
    options.host = "127.0.0.1";
  }
  if (commander.pass) {
    options.password = commander.pass;
  }
  if (port) {
    options.port = +port;
  }
  return fe = require("../" + srv + "/fe")(options);
});
commander.command('link <user/project> <path>').description('  point a project path to somewhere on the file system').action(function(projectName, projectPath) {
  return Project.lookup(projectName, function(project) {
    var exists;
    exists = projectPath && path.existsSync(projectPath);
    if (project && exists) {
      project.path = projectPath;
      return project.save(function(success) {
        if (!success) {
          console.log("Something went wrong:", project.errors);
        }
        return process.exit();
      });
    } else {
      console.log("");
      if (!project) {
        console.log("  * No project at '" + projectName + "'");
      }
      if (!exists) {
        console.log("  * No directory at '" + projectPath + "'");
      }
      console.log("\n  Usage:\n    \n    stratus link username/projectname /path/to/project\n");
      return process.exit();
    }
  });
});
commander.command('show <user/project>').description('  print the path to the project').action(function(projectName) {
  return Project.lookup(projectName, function(project) {
    console.log("");
    if (project) {
      console.log("" + project.path);
    } else {
      if (!project) {
        console.log("  * No project at '" + projectName + "'");
      }
    }
    console.log("");
    return process.exit();
  });
});
commander.command('plugin:install <username> <path>').description('  install the plugin at `path`').action(function(username, pluginPath) {
  var extensions;
  extensions = require("../" + srv + "/fe/models/extensions");
  return Project.lookup("" + username + "/.stratus", function(project) {
    var pluginRepo;
    pluginRepo = extensions(project.path);
    return pluginRepo.install(pluginPath, function(err) {
      if (err) {
        throw err;
      }
      console.log("\n  * Plugin successfully installed.\n");
      return process.exit();
    });
  });
});
commander.parse(process.argv);