(function() {
  var Project, bundle, fs;
  fs = require('fs');
  Project = require('../models').Project;
  bundle = require('stratus-bundle');
  module.exports = function(app) {
    app.post("/projects", function(req, res) {
      var project, projectAttrs, user;
      user = req.user;
      if (!user) {
        req.flash("error", "You need to be logged in to create a project");
        res.redirect("/");
        return;
      }
      projectAttrs = req.param("project");
      projectAttrs.user_id = user.id;
      projectAttrs.isGit = !!projectAttrs.isGit;
      projectAttrs.isPublic = !!projectAttrs.isPublic;
      project = new Project(projectAttrs);
      return project.save(function(success) {
        if (success) {
          req.flash("success", "Project created!");
          return res.redirect("/");
        } else {
          req.flash("error", "Something went wrong...");
          return res.redirect("/");
        }
      });
    });
    app.get("/projects/unique", function(req, res) {
      var name, query;
      name = req.query.name;
      query = {
        name: name,
        user_id: req.user.id
      };
      return Project.where(query, function(projects) {
        query.unique = !projects.length;
        return res.json(query);
      });
    });
    return app.get("/:username/:project", function(req, res, next) {
      var projectName, username, _ref;
      username = req.param("username");
      projectName = req.param("project");
      if (((_ref = req.user) != null ? _ref.name : void 0) !== username) {
        req.flash("error", "You are not authorized to access '" + username + "/" + projectName + "'");
        if (req.user) {
          res.redirect("/");
        } else {
          res.redirect("/login");
        }
        return;
      }
      return Project.lookup("" + username + "/" + projectName, function(project) {
        if (!project) {
          return next();
        }
        return project.list("./", function(err, rootFiles) {
          return project.config(function(err, conf) {
            return project.repo().listR("./", function(allFiles) {
              return bundle.list(function(err, bundles) {
                return res.render("stratus", {
                  layout: false,
                  title: "" + username + "/" + projectName + " - Stratus",
                  data: {
                    project: {
                      path: "" + username + "/" + projectName,
                      files: rootFiles,
                      open: {},
                      lsR: allFiles
                    },
                    config: conf,
                    bundles: bundles.sort()
                  }
                });
              });
            }, true);
          });
        });
      });
    });
  };
}).call(this);
