(function() {
  var Path, PluginRepo, coffee, exec, extensions, fs, loadCSSPlugin, loadJSPlugin, nib, stylus, _;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  fs = require('fs');
  Path = require('path');
  coffee = require('coffee-script');
  stylus = require('stylus');
  nib = require('nib');
  exec = require('child_process').exec;
  _ = require('underscore');
  module.exports = extensions = function(pluginsPath) {
    return new PluginRepo(pluginsPath);
  };
  loadJSPlugin = function(pluginPath) {
    var data, isDir;
    isDir = fs.statSync(pluginPath).isDirectory();
    if (isDir) {
      if (Path.existsSync("" + pluginPath + "/index.js")) {
        return loadJSPlugin("" + pluginPath + "/index.js");
      } else if (Path.existsSync("" + pluginPath + "/index.coffee")) {
        return loadJSPlugin("" + pluginPath + "/index.coffee");
      } else {
        return "";
      }
    } else if (/\.coffee$/.test(pluginPath)) {
      data = fs.readFileSync(pluginPath).toString();
      return coffee.compile(data, {
        filename: pluginPath
      });
    } else if (/\.js$/.test(pluginPath)) {
      return fs.readFileSync(pluginPath).toString();
    } else {
      return "";
    }
  };
  loadCSSPlugin = function(pluginPath, callback) {
    var data, isDir;
    isDir = fs.statSync(pluginPath).isDirectory();
    if (isDir) {
      if (Path.existsSync("" + pluginPath + "/index.styl")) {
        return loadCSSPlugin("" + pluginPath + "/index.styl", callback);
      } else if (Path.existsSync("" + pluginPath + "/index.css")) {
        return loadCSSPlugin("" + pluginPath + "/index.css", callback);
      } else {
        return callback(null, "");
      }
    } else if (/\.styl$/.test(pluginPath)) {
      data = fs.readFileSync(pluginPath).toString();
      return stylus(data, {
        filename: pluginPath
      }).include(nib.path).render(callback);
    } else if (/\.css$/.test(pluginPath)) {
      return fs.readFile(pluginPath, function(err, data) {
        return callback(err, data != null ? data.toString() : void 0);
      });
    } else {
      return callback(null, "");
    }
  };
  PluginRepo = (function() {
    function PluginRepo(path) {
      this.path = path;
    }
    PluginRepo.prototype.install = function(pluginPath, callback) {
      var lastPart;
      lastPart = _.last(pluginPath.split("/"));
      return exec("cp -r '" + pluginPath + "' '" + this.path + "/" + lastPart + "'", function(err, stdout, stderr) {
        return callback(err);
      });
    };
    PluginRepo.prototype.js = function(callback) {
      var jsCode;
      jsCode = "";
      return fs.readdir(this.path, __bind(function(err, files) {
        var file, _i, _len;
        if (err) {
          return callback(err);
        }
        for (_i = 0, _len = files.length; _i < _len; _i++) {
          file = files[_i];
          jsCode += loadJSPlugin("" + this.path + "/" + file);
        }
        return callback(null, jsCode);
      }, this));
    };
    PluginRepo.prototype.css = function(callback) {
      var cssCode;
      cssCode = "";
      return fs.readdir(this.path, __bind(function(err, files) {
        var file, i, _i, _len, _results;
        if (err) {
          return callback(err);
        }
        i = files.length;
        _results = [];
        for (_i = 0, _len = files.length; _i < _len; _i++) {
          file = files[_i];
          _results.push(loadCSSPlugin("" + this.path + "/" + file, function(err, css) {
            cssCode += css + "\n";
            i--;
            if (!i) {
              return callback(null, cssCode);
            }
          }));
        }
        return _results;
      }, this));
    };
    return PluginRepo;
  })();
}).call(this);
