(function() {
  var Model, Project, User, bundle, fs, path, repo, validate, _;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  _ = require('underscore');
  fs = require('fs');
  path = require('path');
  bundle = require('stratus-bundle');
  Model = require('./model');
  User = require('./user');
  repo = require('../../repo');
  validate = require('../../../client/shared/validate');
  module.exports = Project = (function() {
    __extends(Project, Model);
    Project.modelName = "projects";
    Project.prototype.modelName = "projects";
    function Project(options) {
      if (options == null) {
        options = {};
      }
      Project.__super__.constructor.call(this, options);
      this.id = options.id, this.name = options.name, this.user_id = options.user_id, this.path = options.path, this.protocol = options.protocol, this.isPublic = options.isPublic, this.isGit = options.isGit;
    }
    Project.prototype.preSave = function(callback) {
      return User.find(this.user_id, __bind(function(user) {
        return repo.fs.create({
          name: this.name,
          owner: user.name,
          path: this.path,
          protocol: this.protocol,
          isPublic: this.isPublic,
          isGit: this.isGit
        }, __bind(function(_repo) {
          this.path = _repo.path;
          return callback();
        }, this));
      }, this));
    };
    Project.prototype.attributes = function() {
      return {
        id: this.id,
        name: this.name,
        user_id: this.user_id,
        path: this.path,
        protocol: this.protocol,
        isPublic: this.isPublic,
        isGit: this.isGit
      };
    };
    Project.prototype.valid = function(callback) {
      this.errors = _.values(validate.project(this));
      if (!this.user_id) {
        this.errors.push("What the... how can there not be a user?!");
      }
      return Project.where({
        name: this.name,
        user_id: this.user_id
      }, __bind(function(projects) {
        if (projects.length && (!this.id || projects[0].id !== this.id)) {
          this.errors.push("You already have a project named " + this.name);
        }
        return callback(this.errors.length === 0);
      }, this));
    };
    Project.prototype.canRead = function(user, callback) {
      return callback(user.id === this.user_id);
    };
    Project.prototype.canWrite = function(user, callback) {
      return callback(user.id === this.user_id);
    };
    Project.prototype.repo = function() {
      if (this.protocol === "fs") {
        return new repo.fs({
          name: this.name,
          path: this.path,
          protocol: this.protocol,
          isGit: this.isGit
        });
      } else {
        throw new Error("Unsupported Project protocol: '" + this.protocol + "'");
      }
    };
    Project.prototype.list = function(dirpath, callback) {
      this.repo().list(dirpath, function(err, files) {
        var file, fileObjs, i, last, _i, _len, _results;
        if (!files.length) {
          return callback(err, []);
        }
        fileObjs = [];
        last = files.length;
        i = 0;
        _results = [];
        for (_i = 0, _len = files.length; _i < _len; _i++) {
          file = files[_i];
          _results.push((function(file) {
            if (_.last(file) === "/") {
              fileObjs.push({
                name: file
              });
              i++;
              if (i === last) {
                return callback(err, fileObjs);
              }
            } else {
              return bundle.identify(file, function(err, language) {
                fileObjs.push({
                  name: file,
                  language: language
                });
                i++;
                if (i === last) {
                  return callback(err, fileObjs);
                }
              });
            }
          })(file));
        }
        return _results;
      });
    };
    Project.prototype.read = function(filepath, callback) {
      return this.repo().read(filepath, function(err, data) {
        if (data != null) {
          return bundle.identify(filepath, data.slice(0, data.indexOf("\n")), function(err, language) {
            return callback(err, {
              data: data,
              language: language
            });
          });
        } else {
          return callback(err, null);
        }
      });
    };
    Project.prototype.config = function(callback) {
      return fs.readFile("" + __dirname + "/../../config.json", __bind(function(err, baseConfig) {
        if (err) {
          return callback(err);
        }
        baseConfig = JSON.parse(baseConfig.toString());
        return this._userConfig(__bind(function(err, userConfig) {
          return this._config(function(err, projConfig) {
            if (err) {
              return callback(err);
            }
            return callback(null, _.extend(baseConfig, userConfig, projConfig));
          });
        }, this));
      }, this));
    };
    Project.prototype._config = function(callback) {
      var confPath;
      confPath = "" + this.path + "/.stratus.json";
      if (!path.existsSync(confPath)) {
        return callback(null, {});
      }
      return fs.readFile(confPath, function(err, projConfig) {
        if (err) {
          return callback(err);
        }
        projConfig = JSON.parse(projConfig.toString());
        return callback(null, projConfig);
      });
    };
    Project.prototype._userConfig = function(callback) {
      return Project.where({
        user_id: this.user_id,
        name: ".stratus"
      }, function(projects) {
        if (!projects[0]) {
          return callback(null, {});
        }
        return fs.readFile(projects[0].path + "/config.json", function(err, data) {
          if (err) {
            return callback(null, {});
          } else {
            return callback(null, JSON.parse(data.toString()));
          }
        });
      });
    };
    Project.lookup = function(projectPath, callback) {
      var projectName, username, _ref;
      _ref = projectPath.split('/'), username = _ref[0], projectName = _ref[1];
      return User.where({
        name: username
      }, function(users) {
        var user;
        if (!users.length) {
          return callback(null);
        }
        user = users[0];
        return Project.where({
          user_id: user.id,
          name: projectName
        }, function(projects) {
          if (!projects.length) {
            return callback(null);
          }
          return callback(projects[0]);
        });
      });
    };
    return Project;
  })();
  (new Project())._db();
}).call(this);
