(function() {
  /*
  Return an app, which is the front end server.
  
      app = require("fe/server")(some options)
  
  Acceptable options include:
  
  host     - "0.0.0.0" by default, for public. For local-only,
             set to "127.0.0.1".
  listen   - Defaults to `true`, but if `false` the server will
             not be started.
  password - If given, HTTP Basic Authentication will be used.
             The password will be the password given, and the username is
             "stratus" (all lower-case).
  port     - [8134]
  
  */  var NODE_ENV, Path, STATIC_DIR, browserify, bundle, bundleStratus, compileStylus, controllers, express, fractus, helpers, io, nib, sessionStore, stratusUI, stylus, websocket;
  Path = require('path');
  express = require('express');
  io = require('socket.io');
  stylus = require('stylus');
  nib = require('nib');
  browserify = require('browserify');
  fractus = require('fractus');
  controllers = require('./controllers');
  websocket = require('./websocket');
  helpers = require('./helpers');
  NODE_ENV = process.env.NODE_ENV;
  NODE_ENV || (NODE_ENV = "test");
  sessionStore = new express.session.MemoryStore();
  bundle = browserify({
    watch: true
  });
  bundle.addEntry("" + __dirname + "/../../client/site-main.coffee");
  bundleStratus = browserify({
    watch: true,
    mount: "/stratus-browserify.js"
  });
  bundleStratus.addEntry("" + __dirname + "/../../client/stratus-main.coffee");
  STATIC_DIR = "" + __dirname + "/../../public/";
  module.exports = function(options) {
    var app, host, host_s, listen, password, port;
    if (options == null) {
      options = {};
    }
    host = options.host || void 0;
    if (options.listen == null) {
      listen = true;
    }
    password = options.password || false;
    port = options.port || 8134;
    app = express.createServer();
    if (listen) {
      io = io.listen(app);
      if (password) {
        io.configure(function() {
          return io.set('transports', ['xhr-polling', 'websocket']);
        });
      }
    }
    app.configure(function() {
      this.set("views", "" + __dirname + "/../../views");
      this.set("view engine", "jade");
      this.set("view options", {
        title: "Stratus Editor",
        data: {}
      });
      this.use(express.bodyParser());
      this.use(express.methodOverride());
      this.use(express.cookieParser());
      this.use(express.session({
        secret: "im not wearing pants",
        key: "express.sid",
        store: sessionStore
      }));
      if (password) {
        this.use(express.basicAuth("stratus", password));
      }
      this.use(stylus.middleware({
        compile: compileStylus,
        src: STATIC_DIR
      }));
      this.use(express.compiler({
        enable: ["coffeescript"],
        src: STATIC_DIR
      }));
      this.use(bundle);
      this.use(bundleStratus);
      this.use(express.static(STATIC_DIR));
      require('./controllers/auth')(app);
      return this.use(app.router);
    });
    app.configure("development", function() {
      return this.use(express.errorHandler({
        dumpExceptions: true,
        showStack: true
      }));
    });
    app.configure("production", function() {
      return this.use(express.errorHandler());
    });
    helpers(app);
    controllers(app, options);
    if (listen) {
      websocket(io, sessionStore);
    }
    app.get("*", function(req, res) {
      return res.render("404", {
        status: 404
      });
    });
    if (listen) {
      app.listen(port, host);
      host_s = host || "0.0.0.0";
      if (NODE_ENV !== "test") {
        console.log(" * Stratus is listening at http://" + host_s + ":" + port + "/");
      }
    }
    return app;
  };
  stratusUI = Path.dirname(require.resolve("stratus-ui")) + "/css";
  compileStylus = function(str, path) {
    return stylus(str).set("filename", path).include(nib.path).include(stratusUI).include(fractus.path);
  };
}).call(this);
