(function() {
  /*
  Use socket.io to give the client access file system operations on the repo.
  
  TODO: handle permissions
  */  var Project, bundle, getUser, setup;
  bundle = require('stratus-bundle');
  Project = require('../models').Project;
  getUser = require('./helpers').getUser;
  setup = function(socket, data, callback) {
    return getUser(socket, function(err, user) {
      return Project.lookup(data.project, function(project) {
        return callback(user, project, project.repo());
      });
    });
  };
  module.exports = function(io, sessionStore) {
    var fsIo;
    return fsIo = io.of("/fs").on("connection", function(socket) {
      var basicActions, event, _fn, _i, _len;
      basicActions = ["delete", "exists", "mkdir", "isDirectory"];
      _fn = function(event) {
        return socket.on(event, function(data, callback) {
          return setup(socket, data, function(user, project, repo) {
            return repo[event](data.path, callback);
          });
        });
      };
      for (_i = 0, _len = basicActions.length; _i < _len; _i++) {
        event = basicActions[_i];
        _fn(event);
      }
      socket.on("touch", function(data, callback) {
        return setup(socket, data, function(user, project, repo) {
          return repo.touch(data.path, function() {
            return bundle.identify(data.path, function(err, language) {
              return callback({
                language: language
              });
            });
          });
        });
      });
      socket.on("list", function(data, callback) {
        return setup(socket, data, function(user, project, repo) {
          return project.list(data.path, function(err, files) {
            return callback(files, err);
          });
        });
      });
      socket.on("read", function(data, callback) {
        return setup(socket, data, function(user, project, repo) {
          return project.read(data.path, function(err, file) {
            return callback(file, err);
          });
        });
      });
      socket.on("write", function(data, callback) {
        return setup(socket, data, function(user, project, repo) {
          return repo.write(data.path, data.data, function(err) {
            return callback(err);
          });
        });
      });
      socket.on("move", function(data, callback) {
        return setup(socket, data, function(user, project, repo) {
          return repo.move(data.fromPath, data.toPath, function(err) {
            return bundle.identify(data.toPath, function(__, language) {
              return callback(err, {
                language: language
              });
            });
          });
        });
      });
      return socket.on("copy", function(data, callback) {
        return setup(socket, data, function(user, project, repo) {
          return repo.copy(data.fromPath, data.toPath, function(err) {
            return bundle.identify(data.toPath, function(__, language) {
              return callback(err, {
                language: language
              });
            });
          });
        });
      });
    });
  };
}).call(this);
