(function() {
  /*
  TODO
  * dont let files be created with whitespace names
  
  */  var FS_BASE, FsRepo, Repo, STRATUS, allocate, cleanPath, createFsRepo, exec, filename, findit, fs, path, util;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  fs = require('fs');
  path = require('path');
  util = require('util');
  exec = require('child_process').exec;
  findit = require('findit');
  Repo = require('./base_repo');
  STRATUS = path.resolve(process.env.HOME, ".stratus");
  FS_BASE = path.resolve(STRATUS, process.env.NODE_ENV || "development", "projects");
  if (!path.existsSync(STRATUS)) {
    fs.mkdirSync(STRATUS, 0755);
  }
  if (!path.existsSync(FS_BASE)) {
    fs.mkdirSync(FS_BASE, 0755);
  }
  createFsRepo = function(options, callback) {
    var projectPath;
    if (options.isGit) {
      throw new Error("Not implemented.");
    } else if (options.path) {
      return callback(new FsRepo({
        name: options.name,
        path: options.path,
        protocol: "fs",
        isGit: false
      }));
    } else if (options.isGit === false) {
      projectPath = allocate(options.owner, options.name);
      return callback(new FsRepo({
        name: options.name,
        path: projectPath,
        protocol: "fs",
        isGit: false
      }));
    } else if (options.clone) {
      throw new Error("Not implemented.");
    }
  };
  allocate = function(owner, projectName) {
    var projectDir, userDir;
    userDir = path.resolve("" + FS_BASE + "/" + owner);
    projectDir = path.resolve(userDir, projectName);
    if (!path.existsSync(userDir)) {
      fs.mkdirSync(userDir, 0755);
    }
    fs.mkdirSync(projectDir, 0755);
    return projectDir;
  };
  module.exports = FsRepo = (function() {
    __extends(FsRepo, Repo);
    function FsRepo() {
      FsRepo.__super__.constructor.apply(this, arguments);
    }
    FsRepo.create = createFsRepo;
    FsRepo.prototype["delete"] = function(filepath, callback) {
      var fullPath;
      fullPath = this._clean(filepath, filepath);
      if (this.isDirectorySync(filepath)) {
        return this.list(filepath, __bind(function(err, files) {
          var file, _i, _len;
          for (_i = 0, _len = files.length; _i < _len; _i++) {
            file = files[_i];
            this["delete"]("" + filepath + "/" + file);
          }
          fs.rmdirSync(fullPath);
          return typeof callback === "function" ? callback() : void 0;
        }, this));
      } else {
        fs.unlinkSync(fullPath);
        return typeof callback === "function" ? callback() : void 0;
      }
    };
    FsRepo.prototype.move = function(fromPath, toPath, callback) {
      return fs.rename(this._clean(fromPath), this._clean(toPath), callback);
    };
    FsRepo.prototype.copy = function(fromPath, toPath, callback) {
      return exec("cp -r '" + (this._clean(fromPath)) + "' '" + (this._clean(toPath)) + "'", callback);
    };
    FsRepo.prototype.exists = function(filepath, callback) {
      return callback(path.existsSync(this._clean(filepath)));
    };
    FsRepo.prototype.mkdir = function(dirpath, callback) {
      var directory;
      directory = this._clean(dirpath);
      if (path.existsSync(directory)) {
        return typeof callback === "function" ? callback() : void 0;
      }
      return fs.mkdir(directory, 0755, function() {
        return typeof callback === "function" ? callback() : void 0;
      });
    };
    FsRepo.prototype.list = function(dirpath, callback) {
      var directory;
      directory = this._clean(dirpath);
      return fs.readdir(directory, __bind(function(err, files) {
        var file, newFiles, _i, _len;
        newFiles = [];
        for (_i = 0, _len = files.length; _i < _len; _i++) {
          file = files[_i];
          if (this.isDirectorySync(path.join(dirpath, file))) {
            newFiles.push(file + "/");
          } else {
            newFiles.push(file);
          }
        }
        return callback(err, newFiles);
      }, this));
    };
    FsRepo.prototype.listR = function(dirpath, callback, excludeDirs) {
      var file, files, isDir, newFiles, _i, _len;
      if (excludeDirs == null) {
        excludeDirs = false;
      }
      files = findit.sync(this._clean(dirpath));
      newFiles = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        file = file.slice(this.path.length);
        if (file[0] === "/") {
          file = file.slice(1);
        }
        isDir = this.isDirectorySync(file);
        if (excludeDirs) {
          if (!isDir) {
            newFiles.push(file);
          }
        } else if (isDir) {
          newFiles.push(file + "/");
        } else {
          newFiles.push(file);
        }
      }
      return callback(newFiles);
    };
    FsRepo.prototype.isDirectory = function(dirpath, callback) {
      return callback(fs.statSync(this._clean(dirpath)).isDirectory());
    };
    FsRepo.prototype.isDirectorySync = function(dirpath) {
      try {
        return fs.statSync(this._clean(dirpath)).isDirectory();
      } catch (err) {
        return false;
      }
    };
    FsRepo.prototype.touch = function(file, callback) {
      var fullPath;
      fullPath = this._clean(file);
      if (path.existsSync(fullPath)) {
        return typeof callback === "function" ? callback() : void 0;
      }
      return fs.writeFile(fullPath, "", function() {
        return typeof callback === "function" ? callback() : void 0;
      });
    };
    FsRepo.prototype.read = function(file, callback) {
      var fullPath;
      fullPath = this._clean(file);
      return fs.readFile(fullPath, function(err, data) {
        return callback(err, data != null ? data.toString() : void 0);
      });
    };
    FsRepo.prototype.write = function(file, data, callback) {
      var fullPath;
      fullPath = this._clean(file);
      return fs.writeFile(fullPath, data, callback);
    };
    FsRepo.prototype._clean = function(filepath) {
      return cleanPath(this.path, filepath);
    };
    return FsRepo;
  })();
  filename = function(filePath) {
    return path.basename(path.normalize(filePath));
  };
  cleanPath = function(projectRoot, filePath) {
    var clean;
    if (!filePath) {
      throw new Error("You can't do that!");
    }
    if (projectRoot !== "/") {
      projectRoot = projectRoot.replace(/[\/]$/, "");
    }
    clean = path.join(projectRoot, path.normalize(filePath));
    if (clean.indexOf(projectRoot + "/") === 0) {
      return clean;
    } else {
      throw new Error("You can't do that!");
    }
  };
}).call(this);
